from GoodCalculator.calculator import Calculator
import sys

def run_cli():
	calc = Calculator()

	equ = ''
	for arg in sys.argv[1:]:
		equ += arg

	if equ == '':
		print("Empty input")
	else:
		calc.parse_equation(equ)
		calc.evaluate()

if __name__ == "__main__":
	run_cli()


############################################
#	Operator object
############################################
class Operator():
	def __init__(self):
		self.binary_operator = ['<add>','<subtract>','<multiply>','<divide>']
		self.unary_operator = ['<backspace>','<reciprocal>','<square>','<sqrt>','<percentage>']
		self.equal_operator = ['<equal>']

	def op_type(self, op):
		if op in self.binary_operator:
			return 'binary'
		elif op in self.unary_operator:
			return 'unary'
		elif op in self.equal_operator:
			return 'equal'
		else:
			return None

	def binary_op_evaluate(self, op, left, right):
		if op == '<add>':
			return left + right
		elif op == '<subtract>':
			return left - right
		elif op == '<multiply>':
			return left * right
		elif op == '<divide>':
			return left / right

	def unary_op_evaluate(self, op, left):
		if op == '<backspace>':
			return float(int(left/10))
		elif op == '<reciprocal>':
			return 1 / left
		elif op == '<square>':
			return left ** 2
		elif op == '<sqrt>':
			return left ** (1/2)
		elif op == '<percentage>':
			return left / 100





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    # Some information about this package
    name = "GoodCalculator",
    version = "0.0.5",
    author = "Joey",
    author_email = "test@gmail.com",
    description = "A calculator",
    long_description = long_description, # use README.md or README.rst file
    long_description_content_type = "text/markdown", # text/plain | text/x-rst | text/markdown
    url = "https://github.com/pypa/sampleproject", # this will get some statics from github
    # Standard desciptions for this package to let community members to find projects based on their desired criteria. 
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

    # Python requirements
    python_requires = '>=3.6',
    
    # Tell distutils where are the packages (find __init__.py file)
    # packages=setuptools.find_packages(),
    # or use:
    packages = [
        "GoodCalculator"
    ],
    # Where to find packages
    package_dir = [],

    # Entry point to use command line
    # you can decide your dir (GoodCalculator_cli) whatever you want
    # <command in console> = <module name>.<some kind of script>:<function>
    entry_points={
        "console_scripts": [
            "cal = GoodCalculator.GoodCalculator_cli:run_cli"
        ]
    },

    # Install requirements
    install_requires = [],
    # Or, from other sources like github
    dependency_links = []
)


from GoodCalculator.goodOperator import Operator

############################################
#	Calculator object
############################################
class Calculator():
	def __init__(self):
		self.equation_list = []
		self.equation_result = 0.0
		self.operator_map = [
			['<-', '<backspace>'],
			['1/x', '<reciprocal>'],
			['x^2', '<square>'],
			['sqrt(x)', '<sqrt>'],
			['+', '<add>'],
			['-', '<subtract>'],
			['*', '<multiply>'],
			['/', '<divide>'],
			['%', '<percentage>'],
			['=', '<equal>'],
		]
		self.operator_tag = [self.operator_map[i][1] for i in range(len(self.operator_map))]
		# initialize operator object
		self.operator = Operator()

	def parse_equation(self, equation):
		# replace symbol to tag
		for i in range(len(self.operator_map)):
			equation = equation.replace(self.operator_map[i][0],' '+self.operator_map[i][1]+' ')

		# remove empty string
		self.equation_list = equation.split(' ')
		while('' in self.equation_list): self.equation_list.remove('')

		# oprand to float
		for i in range(len(self.equation_list)):
			if self.equation_list[i] not in self.operator_tag:
				self.equation_list[i] = float(self.equation_list[i])

		# first negative number
		if not isinstance(self.equation_list[0], float):
			self.equation_list.pop(0)
			self.equation_list[0] = -self.equation_list[0]

		# negative sign after binary operator and right in the equation
		del_operator_idx = []
		for i in range(len(self.operator.binary_operator)):
			for j in range(len(self.equation_list)):
				if self.operator.binary_operator[i] == self.equation_list[j] and self.equation_list[j+1] == '<subtract>':
					del_operator_idx.append(j+1)
					self.equation_list[j+2] = -self.equation_list[j+2]
		temp = []
		for i in range(len(self.equation_list)):
			if i not in del_operator_idx:
				temp.append(self.equation_list[i])
		self.equation_list = temp

	def evaluate(self):
		# get oprand and oprator list
		oprand_list = []
		oprator_list = []
		for e in self.equation_list:
			if isinstance(e, float):
				oprand_list.append(e)
			else:
				oprator_list.append(e)

		# evaluate result
		for op in oprator_list:
			if self.operator.op_type(op) == 'binary':
				result = self.operator.binary_op_evaluate(op,oprand_list.pop(0),oprand_list.pop(0))
				oprand_list.insert(0, result)
			elif self.operator.op_type(op) == 'unary':
				result = self.operator.unary_op_evaluate(op,oprand_list.pop(0))
				oprand_list.insert(0, result)
			elif self.operator.op_type(op) == 'equal':
				pass
			else:
				raise ValueError("Invalid oprator!")
		
		return oprand_list[0]

